Package: mlr
Title: Machine Learning in R
Version: 2.19.3
Authors@R: c(
    person("Bernd", "Bischl", , "bernd_bischl@gmx.net", role = "aut",
           comment = c(ORCID = "0000-0001-6002-6980")),
    person("Michel", "Lang", , "michellang@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-9754-0393")),
    person("Lars", "Kotthoff", , "larsko@uwyo.edu", role = "aut"),
    person("Patrick", "Schratz", , "patrick.schratz@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-0748-6624")),
    person("Julia", "Schiffner", , "schiffner@math.uni-duesseldorf.de", role = "aut"),
    person("Jakob", "Richter", , "code@jakob-r.de", role = "aut"),
    person("Zachary", "Jones", , "zmj@zmjones.com", role = "aut"),
    person("Giuseppe", "Casalicchio", , "giuseppe.casalicchio@stat.uni-muenchen.de", role = "aut",
           comment = c(ORCID = "0000-0001-5324-5966")),
    person("Mason", "Gallo", , "masonagallo@gmail.com", role = "aut"),
    person("Jakob", "Bossek", , "jakob.bossek@tu-dortmund.de", role = "ctb",
           comment = c(ORCID = "0000-0002-4121-4668")),
    person("Erich", "Studerus", , "erich.studerus@upkbs.ch", role = "ctb",
           comment = c(ORCID = "0000-0003-4233-0182")),
    person("Leonard", "Judt", , "leonard.judt@tu-dortmund.de", role = "ctb"),
    person("Tobias", "Kuehn", , "tobi.kuehn@gmx.de", role = "ctb"),
    person("Pascal", "Kerschke", , "kerschke@uni-muenster.de", role = "ctb",
           comment = c(ORCID = "0000-0003-2862-1418")),
    person("Florian", "Fendt", , "flo_fendt@gmx.de", role = "ctb"),
    person("Philipp", "Probst", , "philipp_probst@gmx.de", role = "ctb",
           comment = c(ORCID = "0000-0001-8402-6790")),
    person("Xudong", "Sun", , "xudong.sun@stat.uni-muenchen.de", role = "ctb",
           comment = c(ORCID = "0000-0003-3269-2307")),
    person("Janek", "Thomas", , "janek.thomas@stat.uni-muenchen.de", role = "ctb",
           comment = c(ORCID = "0000-0003-4511-6245")),
    person("Bruno", "Vieira", , "bruno.hebling.vieira@usp.br", role = "ctb"),
    person("Laura", "Beggel", , "laura.beggel@web.de", role = "ctb",
           comment = c(ORCID = "0000-0002-8872-8535")),
    person("Quay", "Au", , "quay.au@stat.uni-muenchen.de", role = "ctb",
           comment = c(ORCID = "0000-0002-5252-8902")),
    person("Martin", "Binder", , "mlr.developer@mb706.com", role = c("aut", "cre")),
    person("Florian", "Pfisterer", , "pfistererf@googlemail.com", role = "ctb"),
    person("Stefan", "Coors", , "stefan.coors@gmx.net", role = "ctb"),
    person("Steve", "Bronder", , "sab2287@columbia.edu", role = "ctb"),
    person("Alexander", "Engelhardt", , "alexander.w.engelhardt@gmail.com", role = "ctb"),
    person("Christoph", "Molnar", , "christoph.molnar@stat.uni-muenchen.de", role = "ctb"),
    person("Annette", "Spooner", , "a.spooner@unsw.edu.au", role = "ctb")
  )
Description: Interface to a large number of classification and regression
    techniques, including machine-readable parameter descriptions. There
    is also an experimental extension for survival analysis, clustering
    and general, example-specific cost-sensitive learning. Generic
    resampling, including cross-validation, bootstrapping and subsampling.
    Hyperparameter tuning with modern optimization techniques, for single-
    and multi-objective problems.  Filter and wrapper methods for feature
    selection. Extension of basic learners with additional operations
    common in machine learning, also allowing for easy nested resampling.
    Most operations can be parallelized.
License: BSD_2_clause + file LICENSE
URL: https://mlr.mlr-org.com, https://github.com/mlr-org/mlr
BugReports: https://github.com/mlr-org/mlr/issues
Depends: ParamHelpers (>= 1.10), R (>= 3.0.2)
Imports: backports (>= 1.1.0), BBmisc (>= 1.11), checkmate (>= 1.8.2),
        data.table (>= 1.12.4), ggplot2, methods, parallelMap (>= 1.3),
        stats, stringi, survival, utils, XML
Suggests: ada, adabag, batchtools, bit64, brnn, bst, C50, care, caret
        (>= 6.0-57), class, clue, cluster, ClusterR, clusterSim (>=
        0.44-5), cmaes, cowplot, crs, Cubist, deepnet, DiceKriging,
        e1071, earth, elasticnet, emoa, evtree, fda.usc, FDboost, FNN,
        forecast (>= 8.3), fpc, frbs, FSelector, FSelectorRcpp (>=
        0.3.5), gbm, GenSA, ggpubr, glmnet, GPfit, h2o (>= 3.6.0.8),
        Hmisc, irace (>= 2.0), kernlab, kknn, klaR, knitr, laGP,
        LiblineaR, lintr (>= 1.0.0.9001), MASS, mboost, mco, mda,
        memoise, mlbench, mldr, mlrMBO, modeltools, mRMRe, neuralnet,
        nnet, numDeriv, pamr, pander, party, pec, penalized (>=
        0.9-47), pls, PMCMRplus, praznik (>= 5.0.0), randomForest,
        ranger (>= 0.8.0), rappdirs, refund, rex, rFerns, rgenoud,
        rmarkdown, Rmpi, ROCR, rotationForest, rpart, RRF, rsm, RSNNS,
        rucrdtw, RWeka, sda, sf, smoof, sparseLDA, stepPlr, survAUC,
        svglite, testthat, tgp, TH.data, tidyr, tsfeatures, vdiffr,
        wavelets, xgboost (>= 0.7)
VignetteBuilder: knitr
ByteCompile: yes
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first:
        featsel_plotFilterValues,base_plotResiduals,base_generateHyperParsEffect,
        tune_tuneIrace, featsel_filters, learners_all*, regr_h2ogbm
Encoding: UTF-8
LazyData: yes
RoxygenNote: 7.3.2
SystemRequirements: gdal (optional), geos (optional), proj (optional),
        udunits (optional), gsl (optional), gmp (optional), glu
        (optional), jags (optional), mpfr (optional), openmpi
        (optional)
NeedsCompilation: yes
Packaged: 2025-08-22 20:16:32 UTC; user
Author: Bernd Bischl [aut] (ORCID: <https://orcid.org/0000-0001-6002-6980>),
  Michel Lang [aut] (ORCID: <https://orcid.org/0000-0001-9754-0393>),
  Lars Kotthoff [aut],
  Patrick Schratz [aut] (ORCID: <https://orcid.org/0000-0003-0748-6624>),
  Julia Schiffner [aut],
  Jakob Richter [aut],
  Zachary Jones [aut],
  Giuseppe Casalicchio [aut] (ORCID:
    <https://orcid.org/0000-0001-5324-5966>),
  Mason Gallo [aut],
  Jakob Bossek [ctb] (ORCID: <https://orcid.org/0000-0002-4121-4668>),
  Erich Studerus [ctb] (ORCID: <https://orcid.org/0000-0003-4233-0182>),
  Leonard Judt [ctb],
  Tobias Kuehn [ctb],
  Pascal Kerschke [ctb] (ORCID: <https://orcid.org/0000-0003-2862-1418>),
  Florian Fendt [ctb],
  Philipp Probst [ctb] (ORCID: <https://orcid.org/0000-0001-8402-6790>),
  Xudong Sun [ctb] (ORCID: <https://orcid.org/0000-0003-3269-2307>),
  Janek Thomas [ctb] (ORCID: <https://orcid.org/0000-0003-4511-6245>),
  Bruno Vieira [ctb],
  Laura Beggel [ctb] (ORCID: <https://orcid.org/0000-0002-8872-8535>),
  Quay Au [ctb] (ORCID: <https://orcid.org/0000-0002-5252-8902>),
  Martin Binder [aut, cre],
  Florian Pfisterer [ctb],
  Stefan Coors [ctb],
  Steve Bronder [ctb],
  Alexander Engelhardt [ctb],
  Christoph Molnar [ctb],
  Annette Spooner [ctb]
Maintainer: Martin Binder <mlr.developer@mb706.com>
Repository: CRAN
Date/Publication: 2025-08-22 21:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 22:45:04 UTC; unix
Archs: mlr.so.dSYM
