# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,BenchmarkResult)
S3method(as.data.frame,Prediction)
S3method(asROCRPrediction,Prediction)
S3method(asROCRPrediction,ResamplePrediction)
S3method(capLargeValues,Task)
S3method(capLargeValues,data.frame)
S3method(createDummyFeatures,Task)
S3method(createDummyFeatures,character)
S3method(createDummyFeatures,data.frame)
S3method(createDummyFeatures,factor)
S3method(downsample,ResampleInstance)
S3method(downsample,Task)
S3method(estimateRelativeOverfitting,Prediction)
S3method(estimateRelativeOverfitting,ResampleDesc)
S3method(estimateRelativeOverfitting,ResamplePrediction)
S3method(estimateResidualVariance,Learner)
S3method(estimateResidualVariance,WrappedModel)
S3method(extractFDAFeatures,Task)
S3method(extractFDAFeatures,data.frame)
S3method(generateCalibrationData,BenchmarkResult)
S3method(generateCalibrationData,Prediction)
S3method(generateCalibrationData,ResampleResult)
S3method(generateCalibrationData,list)
S3method(generateThreshVsPerfData,BenchmarkResult)
S3method(generateThreshVsPerfData,Prediction)
S3method(generateThreshVsPerfData,ResampleResult)
S3method(generateThreshVsPerfData,list)
S3method(getClassWeightParam,BaseEnsemble)
S3method(getClassWeightParam,BaseWrapper)
S3method(getClassWeightParam,Learner)
S3method(getClassWeightParam,character)
S3method(getFailureModelDump,BaseWrapperModel)
S3method(getFailureModelDump,FailureModel)
S3method(getFailureModelDump,HomogeneousEnsembleModel)
S3method(getFailureModelDump,WrappedModel)
S3method(getFailureModelMsg,BaseWrapperModel)
S3method(getFailureModelMsg,FailureModel)
S3method(getFailureModelMsg,HomogeneousEnsembleModel)
S3method(getFailureModelMsg,WrappedModel)
S3method(getFeatureImportanceLearner,BaseWrapper)
S3method(getFeatureImportanceLearner,classif.RRF)
S3method(getFeatureImportanceLearner,classif.boosting)
S3method(getFeatureImportanceLearner,classif.cforest)
S3method(getFeatureImportanceLearner,classif.gbm)
S3method(getFeatureImportanceLearner,classif.h2o.deeplearning)
S3method(getFeatureImportanceLearner,classif.h2o.gbm)
S3method(getFeatureImportanceLearner,classif.h2o.glm)
S3method(getFeatureImportanceLearner,classif.h2o.randomForest)
S3method(getFeatureImportanceLearner,classif.randomForest)
S3method(getFeatureImportanceLearner,classif.ranger)
S3method(getFeatureImportanceLearner,classif.rpart)
S3method(getFeatureImportanceLearner,classif.xgboost)
S3method(getFeatureImportanceLearner,regr.RRF)
S3method(getFeatureImportanceLearner,regr.cforest)
S3method(getFeatureImportanceLearner,regr.gbm)
S3method(getFeatureImportanceLearner,regr.randomForest)
S3method(getFeatureImportanceLearner,regr.ranger)
S3method(getFeatureImportanceLearner,regr.rpart)
S3method(getFeatureImportanceLearner,regr.xgboost)
S3method(getFeatureImportanceLearner,surv.cforest)
S3method(getFeatureImportanceLearner,surv.gbm)
S3method(getFeatureImportanceLearner,surv.ranger)
S3method(getFeatureImportanceLearner,surv.rpart)
S3method(getFilteredFeatures,FilterModel)
S3method(getFilteredFeatures,default)
S3method(getFunctionalFeatures,Task)
S3method(getFunctionalFeatures,data.frame)
S3method(getHyperPars,BaseEnsemble)
S3method(getHyperPars,BaseWrapper)
S3method(getHyperPars,Learner)
S3method(getLearnerModel,BaseWrapperModel)
S3method(getLearnerModel,HomogeneousEnsembleModel)
S3method(getLearnerModel,ModelMultiplexerModel)
S3method(getLearnerModel,WrappedModel)
S3method(getLearnerProperties,BaggingWrapper)
S3method(getLearnerProperties,BaseWrapper)
S3method(getLearnerProperties,ClassificationViaRegressionWrapper)
S3method(getLearnerProperties,CostSensClassifWrapper)
S3method(getLearnerProperties,CostSensRegrWrapper)
S3method(getLearnerProperties,CostSensWeightedPairsWrapper)
S3method(getLearnerProperties,DummyFeaturesWrapper)
S3method(getLearnerProperties,ImputeWrapper)
S3method(getLearnerProperties,Learner)
S3method(getLearnerProperties,ModelMultiplexer)
S3method(getLearnerProperties,MulticlassWrapper)
S3method(getLearnerProperties,OverBaggingWrapper)
S3method(getLearnerProperties,PreprocWrapperCaret)
S3method(getLearnerProperties,WeightedClassesWrapper)
S3method(getLearnerProperties,character)
S3method(getLearnerProperties,extractFDAFeatsWrapper)
S3method(getOOBPredsLearner,BaseWrapper)
S3method(getOOBPredsLearner,classif.rFerns)
S3method(getOOBPredsLearner,classif.randomForest)
S3method(getOOBPredsLearner,classif.ranger)
S3method(getOOBPredsLearner,regr.randomForest)
S3method(getOOBPredsLearner,regr.ranger)
S3method(getParamSet,BaseWrapper)
S3method(getParamSet,Learner)
S3method(getParamSet,character)
S3method(getPredictionResponse,PredictionMultilabel)
S3method(getPredictionResponse,default)
S3method(getPredictionSE,default)
S3method(getPredictionTruth,PredictionCluster)
S3method(getPredictionTruth,PredictionMultilabel)
S3method(getPredictionTruth,PredictionSurv)
S3method(getPredictionTruth,default)
S3method(getTaskClassLevels,ClassifTask)
S3method(getTaskClassLevels,ClassifTaskDesc)
S3method(getTaskClassLevels,MultilabelTask)
S3method(getTaskClassLevels,MultilabelTaskDesc)
S3method(getTaskCosts,CostSensTask)
S3method(getTaskCosts,Task)
S3method(getTaskDesc,TaskDesc)
S3method(getTaskDesc,default)
S3method(getTaskFeatureNames,Task)
S3method(getTaskTargetNames,SupervisedTaskDesc)
S3method(getTaskTargetNames,Task)
S3method(getTaskTargetNames,UnsupervisedTaskDesc)
S3method(getTaskTargets,CostSensTask)
S3method(getTaskTargets,SupervisedTask)
S3method(getTaskTargets,UnsupervisedTask)
S3method(hasFunctionalFeatures,Task)
S3method(hasFunctionalFeatures,data.frame)
S3method(impute,Task)
S3method(impute,data.frame)
S3method(instantiateResampleInstance,BootstrapDesc)
S3method(instantiateResampleInstance,CVDesc)
S3method(instantiateResampleInstance,CVHelperDesc)
S3method(instantiateResampleInstance,FixedWindowCVDesc)
S3method(instantiateResampleInstance,GrowingWindowCVDesc)
S3method(instantiateResampleInstance,HoldoutDesc)
S3method(instantiateResampleInstance,LOODesc)
S3method(instantiateResampleInstance,RepCVDesc)
S3method(instantiateResampleInstance,SpCVDesc)
S3method(instantiateResampleInstance,SpRepCVDesc)
S3method(instantiateResampleInstance,SubsampleDesc)
S3method(isFailureModel,BaseWrapperModel)
S3method(isFailureModel,ClassificationViaRegressionModel)
S3method(isFailureModel,FailureModel)
S3method(isFailureModel,HomogeneousEnsembleModel)
S3method(isFailureModel,ModelMultiplexerModel)
S3method(isFailureModel,WrappedModel)
S3method(joinClassLevels,ClassifTask)
S3method(listLearners,Task)
S3method(listLearners,character)
S3method(listLearners,default)
S3method(listMeasures,Task)
S3method(listMeasures,character)
S3method(listMeasures,default)
S3method(makePrediction,ClassifTaskDesc)
S3method(makePrediction,ClusterTaskDesc)
S3method(makePrediction,CostSensTaskDesc)
S3method(makePrediction,MultilabelTaskDesc)
S3method(makePrediction,RegrTaskDesc)
S3method(makePrediction,SurvTaskDesc)
S3method(makeRLearner,classif.C50)
S3method(makeRLearner,classif.FDboost)
S3method(makeRLearner,classif.IBk)
S3method(makeRLearner,classif.J48)
S3method(makeRLearner,classif.JRip)
S3method(makeRLearner,classif.LiblineaRL1L2SVC)
S3method(makeRLearner,classif.LiblineaRL1LogReg)
S3method(makeRLearner,classif.LiblineaRL2L1SVC)
S3method(makeRLearner,classif.LiblineaRL2LogReg)
S3method(makeRLearner,classif.LiblineaRL2SVC)
S3method(makeRLearner,classif.LiblineaRMultiClassSVC)
S3method(makeRLearner,classif.OneR)
S3method(makeRLearner,classif.PART)
S3method(makeRLearner,classif.RRF)
S3method(makeRLearner,classif.ada)
S3method(makeRLearner,classif.adaboostm1)
S3method(makeRLearner,classif.binomial)
S3method(makeRLearner,classif.boosting)
S3method(makeRLearner,classif.bst)
S3method(makeRLearner,classif.cforest)
S3method(makeRLearner,classif.clusterSVM)
S3method(makeRLearner,classif.ctree)
S3method(makeRLearner,classif.cvglmnet)
S3method(makeRLearner,classif.dbnDNN)
S3method(makeRLearner,classif.dcSVM)
S3method(makeRLearner,classif.earth)
S3method(makeRLearner,classif.evtree)
S3method(makeRLearner,classif.fdausc.glm)
S3method(makeRLearner,classif.fdausc.kernel)
S3method(makeRLearner,classif.fdausc.knn)
S3method(makeRLearner,classif.fdausc.np)
S3method(makeRLearner,classif.featureless)
S3method(makeRLearner,classif.fgam)
S3method(makeRLearner,classif.fnn)
S3method(makeRLearner,classif.gamboost)
S3method(makeRLearner,classif.gaterSVM)
S3method(makeRLearner,classif.gausspr)
S3method(makeRLearner,classif.gbm)
S3method(makeRLearner,classif.glmboost)
S3method(makeRLearner,classif.glmnet)
S3method(makeRLearner,classif.h2o.deeplearning)
S3method(makeRLearner,classif.h2o.gbm)
S3method(makeRLearner,classif.h2o.glm)
S3method(makeRLearner,classif.h2o.randomForest)
S3method(makeRLearner,classif.kknn)
S3method(makeRLearner,classif.knn)
S3method(makeRLearner,classif.ksvm)
S3method(makeRLearner,classif.lda)
S3method(makeRLearner,classif.logreg)
S3method(makeRLearner,classif.lssvm)
S3method(makeRLearner,classif.lvq1)
S3method(makeRLearner,classif.mda)
S3method(makeRLearner,classif.mlp)
S3method(makeRLearner,classif.multinom)
S3method(makeRLearner,classif.naiveBayes)
S3method(makeRLearner,classif.neuralnet)
S3method(makeRLearner,classif.nnTrain)
S3method(makeRLearner,classif.nnet)
S3method(makeRLearner,classif.pamr)
S3method(makeRLearner,classif.penalized)
S3method(makeRLearner,classif.plr)
S3method(makeRLearner,classif.plsdaCaret)
S3method(makeRLearner,classif.probit)
S3method(makeRLearner,classif.qda)
S3method(makeRLearner,classif.rFerns)
S3method(makeRLearner,classif.randomForest)
S3method(makeRLearner,classif.ranger)
S3method(makeRLearner,classif.rda)
S3method(makeRLearner,classif.rotationForest)
S3method(makeRLearner,classif.rpart)
S3method(makeRLearner,classif.saeDNN)
S3method(makeRLearner,classif.sda)
S3method(makeRLearner,classif.sparseLDA)
S3method(makeRLearner,classif.svm)
S3method(makeRLearner,classif.xgboost)
S3method(makeRLearner,cluster.Cobweb)
S3method(makeRLearner,cluster.EM)
S3method(makeRLearner,cluster.FarthestFirst)
S3method(makeRLearner,cluster.MiniBatchKmeans)
S3method(makeRLearner,cluster.SimpleKMeans)
S3method(makeRLearner,cluster.XMeans)
S3method(makeRLearner,cluster.cmeans)
S3method(makeRLearner,cluster.dbscan)
S3method(makeRLearner,cluster.kkmeans)
S3method(makeRLearner,cluster.kmeans)
S3method(makeRLearner,multilabel.cforest)
S3method(makeRLearner,multilabel.rFerns)
S3method(makeRLearner,regr.FDboost)
S3method(makeRLearner,regr.GPfit)
S3method(makeRLearner,regr.IBk)
S3method(makeRLearner,regr.LiblineaRL2L1SVR)
S3method(makeRLearner,regr.LiblineaRL2L2SVR)
S3method(makeRLearner,regr.RRF)
S3method(makeRLearner,regr.bcart)
S3method(makeRLearner,regr.bgp)
S3method(makeRLearner,regr.bgpllm)
S3method(makeRLearner,regr.blm)
S3method(makeRLearner,regr.brnn)
S3method(makeRLearner,regr.bst)
S3method(makeRLearner,regr.btgp)
S3method(makeRLearner,regr.btgpllm)
S3method(makeRLearner,regr.btlm)
S3method(makeRLearner,regr.cforest)
S3method(makeRLearner,regr.crs)
S3method(makeRLearner,regr.ctree)
S3method(makeRLearner,regr.cubist)
S3method(makeRLearner,regr.cvglmnet)
S3method(makeRLearner,regr.earth)
S3method(makeRLearner,regr.evtree)
S3method(makeRLearner,regr.featureless)
S3method(makeRLearner,regr.fgam)
S3method(makeRLearner,regr.fnn)
S3method(makeRLearner,regr.frbs)
S3method(makeRLearner,regr.gamboost)
S3method(makeRLearner,regr.gausspr)
S3method(makeRLearner,regr.gbm)
S3method(makeRLearner,regr.glm)
S3method(makeRLearner,regr.glmboost)
S3method(makeRLearner,regr.glmnet)
S3method(makeRLearner,regr.h2o.deeplearning)
S3method(makeRLearner,regr.h2o.gbm)
S3method(makeRLearner,regr.h2o.glm)
S3method(makeRLearner,regr.h2o.randomForest)
S3method(makeRLearner,regr.kknn)
S3method(makeRLearner,regr.km)
S3method(makeRLearner,regr.ksvm)
S3method(makeRLearner,regr.laGP)
S3method(makeRLearner,regr.lm)
S3method(makeRLearner,regr.mars)
S3method(makeRLearner,regr.mob)
S3method(makeRLearner,regr.nnet)
S3method(makeRLearner,regr.pcr)
S3method(makeRLearner,regr.penalized)
S3method(makeRLearner,regr.plsr)
S3method(makeRLearner,regr.randomForest)
S3method(makeRLearner,regr.ranger)
S3method(makeRLearner,regr.rpart)
S3method(makeRLearner,regr.rsm)
S3method(makeRLearner,regr.rvm)
S3method(makeRLearner,regr.svm)
S3method(makeRLearner,regr.xgboost)
S3method(makeRLearner,surv.cforest)
S3method(makeRLearner,surv.coxph)
S3method(makeRLearner,surv.cvglmnet)
S3method(makeRLearner,surv.gamboost)
S3method(makeRLearner,surv.gbm)
S3method(makeRLearner,surv.glmboost)
S3method(makeRLearner,surv.glmnet)
S3method(makeRLearner,surv.ranger)
S3method(makeRLearner,surv.rpart)
S3method(makeWrappedModel,BaseEnsemble)
S3method(makeWrappedModel,BaseWrapper)
S3method(makeWrappedModel,Learner)
S3method(makeWrappedModel,ModelMultiplexer)
S3method(makeWrappedModel,TuneWrapper)
S3method(normalizeFeatures,Task)
S3method(normalizeFeatures,data.frame)
S3method(plotResiduals,BenchmarkResult)
S3method(plotResiduals,Prediction)
S3method(predict,WrappedModel)
S3method(predictLearner,BaggingWrapper)
S3method(predictLearner,BaseWrapper)
S3method(predictLearner,ClassificationViaRegressionWrapper)
S3method(predictLearner,ConstantClassWrapper)
S3method(predictLearner,CostSensClassifWrapper)
S3method(predictLearner,CostSensRegrWrapper)
S3method(predictLearner,CostSensWeightedPairsWrapper)
S3method(predictLearner,FeatSelWrapper)
S3method(predictLearner,FilterWrapper)
S3method(predictLearner,ModelMultiplexer)
S3method(predictLearner,MulticlassWrapper)
S3method(predictLearner,MultilabelBinaryRelevanceWrapper)
S3method(predictLearner,MultilabelClassifierChainsWrapper)
S3method(predictLearner,MultilabelDBRWrapper)
S3method(predictLearner,MultilabelNestedStackingWrapper)
S3method(predictLearner,MultilabelStackingWrapper)
S3method(predictLearner,PreprocWrapper)
S3method(predictLearner,StackedLearner)
S3method(predictLearner,TuneWrapper)
S3method(predictLearner,classif.C50)
S3method(predictLearner,classif.FDboost)
S3method(predictLearner,classif.IBk)
S3method(predictLearner,classif.J48)
S3method(predictLearner,classif.JRip)
S3method(predictLearner,classif.LiblineaRL1L2SVC)
S3method(predictLearner,classif.LiblineaRL1LogReg)
S3method(predictLearner,classif.LiblineaRL2L1SVC)
S3method(predictLearner,classif.LiblineaRL2LogReg)
S3method(predictLearner,classif.LiblineaRL2SVC)
S3method(predictLearner,classif.LiblineaRMultiClassSVC)
S3method(predictLearner,classif.OneR)
S3method(predictLearner,classif.PART)
S3method(predictLearner,classif.RRF)
S3method(predictLearner,classif.ada)
S3method(predictLearner,classif.adaboostm1)
S3method(predictLearner,classif.binomial)
S3method(predictLearner,classif.boosting)
S3method(predictLearner,classif.bst)
S3method(predictLearner,classif.cforest)
S3method(predictLearner,classif.clusterSVM)
S3method(predictLearner,classif.ctree)
S3method(predictLearner,classif.cvglmnet)
S3method(predictLearner,classif.dbnDNN)
S3method(predictLearner,classif.dcSVM)
S3method(predictLearner,classif.earth)
S3method(predictLearner,classif.evtree)
S3method(predictLearner,classif.fdausc.glm)
S3method(predictLearner,classif.fdausc.kernel)
S3method(predictLearner,classif.fdausc.knn)
S3method(predictLearner,classif.fdausc.np)
S3method(predictLearner,classif.featureless)
S3method(predictLearner,classif.fgam)
S3method(predictLearner,classif.fnn)
S3method(predictLearner,classif.gamboost)
S3method(predictLearner,classif.gaterSVM)
S3method(predictLearner,classif.gausspr)
S3method(predictLearner,classif.gbm)
S3method(predictLearner,classif.glmboost)
S3method(predictLearner,classif.glmnet)
S3method(predictLearner,classif.h2o.deeplearning)
S3method(predictLearner,classif.h2o.gbm)
S3method(predictLearner,classif.h2o.glm)
S3method(predictLearner,classif.h2o.randomForest)
S3method(predictLearner,classif.kknn)
S3method(predictLearner,classif.knn)
S3method(predictLearner,classif.ksvm)
S3method(predictLearner,classif.lda)
S3method(predictLearner,classif.logreg)
S3method(predictLearner,classif.lssvm)
S3method(predictLearner,classif.lvq1)
S3method(predictLearner,classif.mda)
S3method(predictLearner,classif.mlp)
S3method(predictLearner,classif.multinom)
S3method(predictLearner,classif.naiveBayes)
S3method(predictLearner,classif.neuralnet)
S3method(predictLearner,classif.nnTrain)
S3method(predictLearner,classif.nnet)
S3method(predictLearner,classif.pamr)
S3method(predictLearner,classif.penalized)
S3method(predictLearner,classif.plr)
S3method(predictLearner,classif.plsdaCaret)
S3method(predictLearner,classif.probit)
S3method(predictLearner,classif.qda)
S3method(predictLearner,classif.rFerns)
S3method(predictLearner,classif.randomForest)
S3method(predictLearner,classif.ranger)
S3method(predictLearner,classif.rda)
S3method(predictLearner,classif.rotationForest)
S3method(predictLearner,classif.rpart)
S3method(predictLearner,classif.saeDNN)
S3method(predictLearner,classif.sda)
S3method(predictLearner,classif.sparseLDA)
S3method(predictLearner,classif.svm)
S3method(predictLearner,classif.xgboost)
S3method(predictLearner,cluster.Cobweb)
S3method(predictLearner,cluster.EM)
S3method(predictLearner,cluster.FarthestFirst)
S3method(predictLearner,cluster.MiniBatchKmeans)
S3method(predictLearner,cluster.SimpleKMeans)
S3method(predictLearner,cluster.XMeans)
S3method(predictLearner,cluster.cmeans)
S3method(predictLearner,cluster.dbscan)
S3method(predictLearner,cluster.kkmeans)
S3method(predictLearner,cluster.kmeans)
S3method(predictLearner,multilabel.cforest)
S3method(predictLearner,multilabel.rFerns)
S3method(predictLearner,regr.FDboost)
S3method(predictLearner,regr.GPfit)
S3method(predictLearner,regr.IBk)
S3method(predictLearner,regr.LiblineaRL2L1SVR)
S3method(predictLearner,regr.LiblineaRL2L2SVR)
S3method(predictLearner,regr.RRF)
S3method(predictLearner,regr.bcart)
S3method(predictLearner,regr.bgp)
S3method(predictLearner,regr.bgpllm)
S3method(predictLearner,regr.blm)
S3method(predictLearner,regr.brnn)
S3method(predictLearner,regr.bst)
S3method(predictLearner,regr.btgp)
S3method(predictLearner,regr.btgpllm)
S3method(predictLearner,regr.btlm)
S3method(predictLearner,regr.cforest)
S3method(predictLearner,regr.crs)
S3method(predictLearner,regr.ctree)
S3method(predictLearner,regr.cubist)
S3method(predictLearner,regr.cvglmnet)
S3method(predictLearner,regr.earth)
S3method(predictLearner,regr.evtree)
S3method(predictLearner,regr.featureless)
S3method(predictLearner,regr.fgam)
S3method(predictLearner,regr.fnn)
S3method(predictLearner,regr.frbs)
S3method(predictLearner,regr.gamboost)
S3method(predictLearner,regr.gausspr)
S3method(predictLearner,regr.gbm)
S3method(predictLearner,regr.glm)
S3method(predictLearner,regr.glmboost)
S3method(predictLearner,regr.glmnet)
S3method(predictLearner,regr.h2o.deeplearning)
S3method(predictLearner,regr.h2o.gbm)
S3method(predictLearner,regr.h2o.glm)
S3method(predictLearner,regr.h2o.randomForest)
S3method(predictLearner,regr.kknn)
S3method(predictLearner,regr.km)
S3method(predictLearner,regr.ksvm)
S3method(predictLearner,regr.laGP)
S3method(predictLearner,regr.lm)
S3method(predictLearner,regr.mars)
S3method(predictLearner,regr.mob)
S3method(predictLearner,regr.nnet)
S3method(predictLearner,regr.pcr)
S3method(predictLearner,regr.penalized)
S3method(predictLearner,regr.plsr)
S3method(predictLearner,regr.randomForest)
S3method(predictLearner,regr.ranger)
S3method(predictLearner,regr.rpart)
S3method(predictLearner,regr.rsm)
S3method(predictLearner,regr.rvm)
S3method(predictLearner,regr.svm)
S3method(predictLearner,regr.xgboost)
S3method(predictLearner,surv.cforest)
S3method(predictLearner,surv.coxph)
S3method(predictLearner,surv.cvglmnet)
S3method(predictLearner,surv.gamboost)
S3method(predictLearner,surv.gbm)
S3method(predictLearner,surv.glmboost)
S3method(predictLearner,surv.glmnet)
S3method(predictLearner,surv.ranger)
S3method(predictLearner,surv.rpart)
S3method(print,Aggregation)
S3method(print,BaggingModel)
S3method(print,BaseWrapper)
S3method(print,BenchmarkResult)
S3method(print,ChainModel)
S3method(print,ClassifTask)
S3method(print,ClusterTask)
S3method(print,ConfusionMatrix)
S3method(print,CostSensTask)
S3method(print,FailureModel)
S3method(print,FeatSelControl)
S3method(print,FeatSelResult)
S3method(print,FeatureImportance)
S3method(print,Filter)
S3method(print,FilterEnsemble)
S3method(print,FilterEnsembleMethodsList)
S3method(print,FilterMethodsList)
S3method(print,FilterValues)
S3method(print,FixedWindowCVDesc)
S3method(print,GrowingWindowCVDesc)
S3method(print,HoldoutDesc)
S3method(print,HyperParsEffectData)
S3method(print,ImputationDesc)
S3method(print,Learner)
S3method(print,LearningCurveData)
S3method(print,ListLearners)
S3method(print,Measure)
S3method(print,MultilabelTask)
S3method(print,OptModel)
S3method(print,PartialDependenceData)
S3method(print,Prediction)
S3method(print,ROCMeasures)
S3method(print,RepCVDesc)
S3method(print,ResampleDesc)
S3method(print,ResampleInstance)
S3method(print,ResamplePrediction)
S3method(print,ResampleResult)
S3method(print,SubsampleDesc)
S3method(print,SupervisedTask)
S3method(print,Task)
S3method(print,TuneControl)
S3method(print,TuneMultiCritControl)
S3method(print,TuneMultiCritResult)
S3method(print,TuneResult)
S3method(print,UnsupervisedTask)
S3method(print,WrappedModel)
S3method(print,extractFDAFeatDesc)
S3method(print,mlr.dump)
S3method(reextractFDAFeatures,Task)
S3method(reextractFDAFeatures,data.frame)
S3method(reimpute,Task)
S3method(reimpute,data.frame)
S3method(removeConstantFeatures,Task)
S3method(removeConstantFeatures,data.frame)
S3method(removeHyperPars,BaseEnsemble)
S3method(removeHyperPars,BaseWrapper)
S3method(removeHyperPars,Learner)
S3method(setHyperPars2,BaseEnsemble)
S3method(setHyperPars2,BaseWrapper)
S3method(setHyperPars2,Learner)
S3method(setPredictType,BaggingWrapper)
S3method(setPredictType,BaseEnsemble)
S3method(setPredictType,BaseWrapper)
S3method(setPredictType,ClassificationViaRegressionWrapper)
S3method(setPredictType,Learner)
S3method(setPredictType,StackedLearner)
S3method(summarizeColumns,Task)
S3method(summarizeColumns,data.frame)
S3method(summarizeLevels,Task)
S3method(summarizeLevels,data.frame)
S3method(trainLearner,BaggingWrapper)
S3method(trainLearner,ClassificationViaRegressionWrapper)
S3method(trainLearner,ConstantClassWrapper)
S3method(trainLearner,CostSensClassifWrapper)
S3method(trainLearner,CostSensRegrWrapper)
S3method(trainLearner,CostSensWeightedPairsWrapper)
S3method(trainLearner,DownsampleWrapper)
S3method(trainLearner,FeatSelWrapper)
S3method(trainLearner,FilterWrapper)
S3method(trainLearner,ModelMultiplexer)
S3method(trainLearner,MulticlassWrapper)
S3method(trainLearner,MultilabelBinaryRelevanceWrapper)
S3method(trainLearner,MultilabelClassifierChainsWrapper)
S3method(trainLearner,MultilabelDBRWrapper)
S3method(trainLearner,MultilabelNestedStackingWrapper)
S3method(trainLearner,MultilabelStackingWrapper)
S3method(trainLearner,OverBaggingWrapper)
S3method(trainLearner,OversampleWrapper)
S3method(trainLearner,PreprocWrapper)
S3method(trainLearner,SMOTEWrapper)
S3method(trainLearner,StackedLearner)
S3method(trainLearner,TuneWrapper)
S3method(trainLearner,UndersampleWrapper)
S3method(trainLearner,WeightedClassesWrapper)
S3method(trainLearner,classif.C50)
S3method(trainLearner,classif.FDboost)
S3method(trainLearner,classif.IBk)
S3method(trainLearner,classif.J48)
S3method(trainLearner,classif.JRip)
S3method(trainLearner,classif.LiblineaRL1L2SVC)
S3method(trainLearner,classif.LiblineaRL1LogReg)
S3method(trainLearner,classif.LiblineaRL2L1SVC)
S3method(trainLearner,classif.LiblineaRL2LogReg)
S3method(trainLearner,classif.LiblineaRL2SVC)
S3method(trainLearner,classif.LiblineaRMultiClassSVC)
S3method(trainLearner,classif.OneR)
S3method(trainLearner,classif.PART)
S3method(trainLearner,classif.RRF)
S3method(trainLearner,classif.ada)
S3method(trainLearner,classif.adaboostm1)
S3method(trainLearner,classif.binomial)
S3method(trainLearner,classif.boosting)
S3method(trainLearner,classif.bst)
S3method(trainLearner,classif.cforest)
S3method(trainLearner,classif.clusterSVM)
S3method(trainLearner,classif.ctree)
S3method(trainLearner,classif.cvglmnet)
S3method(trainLearner,classif.dbnDNN)
S3method(trainLearner,classif.dcSVM)
S3method(trainLearner,classif.earth)
S3method(trainLearner,classif.evtree)
S3method(trainLearner,classif.fdausc.glm)
S3method(trainLearner,classif.fdausc.kernel)
S3method(trainLearner,classif.fdausc.knn)
S3method(trainLearner,classif.fdausc.np)
S3method(trainLearner,classif.featureless)
S3method(trainLearner,classif.fgam)
S3method(trainLearner,classif.fnn)
S3method(trainLearner,classif.gamboost)
S3method(trainLearner,classif.gaterSVM)
S3method(trainLearner,classif.gausspr)
S3method(trainLearner,classif.gbm)
S3method(trainLearner,classif.glmboost)
S3method(trainLearner,classif.glmnet)
S3method(trainLearner,classif.h2o.deeplearning)
S3method(trainLearner,classif.h2o.gbm)
S3method(trainLearner,classif.h2o.glm)
S3method(trainLearner,classif.h2o.randomForest)
S3method(trainLearner,classif.kknn)
S3method(trainLearner,classif.knn)
S3method(trainLearner,classif.ksvm)
S3method(trainLearner,classif.lda)
S3method(trainLearner,classif.logreg)
S3method(trainLearner,classif.lssvm)
S3method(trainLearner,classif.lvq1)
S3method(trainLearner,classif.mda)
S3method(trainLearner,classif.mlp)
S3method(trainLearner,classif.multinom)
S3method(trainLearner,classif.naiveBayes)
S3method(trainLearner,classif.neuralnet)
S3method(trainLearner,classif.nnTrain)
S3method(trainLearner,classif.nnet)
S3method(trainLearner,classif.pamr)
S3method(trainLearner,classif.penalized)
S3method(trainLearner,classif.plr)
S3method(trainLearner,classif.plsdaCaret)
S3method(trainLearner,classif.probit)
S3method(trainLearner,classif.qda)
S3method(trainLearner,classif.rFerns)
S3method(trainLearner,classif.randomForest)
S3method(trainLearner,classif.ranger)
S3method(trainLearner,classif.rda)
S3method(trainLearner,classif.rotationForest)
S3method(trainLearner,classif.rpart)
S3method(trainLearner,classif.saeDNN)
S3method(trainLearner,classif.sda)
S3method(trainLearner,classif.sparseLDA)
S3method(trainLearner,classif.svm)
S3method(trainLearner,classif.xgboost)
S3method(trainLearner,cluster.Cobweb)
S3method(trainLearner,cluster.EM)
S3method(trainLearner,cluster.FarthestFirst)
S3method(trainLearner,cluster.MiniBatchKmeans)
S3method(trainLearner,cluster.SimpleKMeans)
S3method(trainLearner,cluster.XMeans)
S3method(trainLearner,cluster.cmeans)
S3method(trainLearner,cluster.dbscan)
S3method(trainLearner,cluster.kkmeans)
S3method(trainLearner,cluster.kmeans)
S3method(trainLearner,multilabel.cforest)
S3method(trainLearner,multilabel.rFerns)
S3method(trainLearner,regr.FDboost)
S3method(trainLearner,regr.GPfit)
S3method(trainLearner,regr.IBk)
S3method(trainLearner,regr.LiblineaRL2L1SVR)
S3method(trainLearner,regr.LiblineaRL2L2SVR)
S3method(trainLearner,regr.RRF)
S3method(trainLearner,regr.bcart)
S3method(trainLearner,regr.bgp)
S3method(trainLearner,regr.bgpllm)
S3method(trainLearner,regr.blm)
S3method(trainLearner,regr.brnn)
S3method(trainLearner,regr.bst)
S3method(trainLearner,regr.btgp)
S3method(trainLearner,regr.btgpllm)
S3method(trainLearner,regr.btlm)
S3method(trainLearner,regr.cforest)
S3method(trainLearner,regr.crs)
S3method(trainLearner,regr.ctree)
S3method(trainLearner,regr.cubist)
S3method(trainLearner,regr.cvglmnet)
S3method(trainLearner,regr.earth)
S3method(trainLearner,regr.evtree)
S3method(trainLearner,regr.featureless)
S3method(trainLearner,regr.fgam)
S3method(trainLearner,regr.fnn)
S3method(trainLearner,regr.frbs)
S3method(trainLearner,regr.gamboost)
S3method(trainLearner,regr.gausspr)
S3method(trainLearner,regr.gbm)
S3method(trainLearner,regr.glm)
S3method(trainLearner,regr.glmboost)
S3method(trainLearner,regr.glmnet)
S3method(trainLearner,regr.h2o.deeplearning)
S3method(trainLearner,regr.h2o.gbm)
S3method(trainLearner,regr.h2o.glm)
S3method(trainLearner,regr.h2o.randomForest)
S3method(trainLearner,regr.kknn)
S3method(trainLearner,regr.km)
S3method(trainLearner,regr.ksvm)
S3method(trainLearner,regr.laGP)
S3method(trainLearner,regr.lm)
S3method(trainLearner,regr.mars)
S3method(trainLearner,regr.mob)
S3method(trainLearner,regr.nnet)
S3method(trainLearner,regr.pcr)
S3method(trainLearner,regr.penalized)
S3method(trainLearner,regr.plsr)
S3method(trainLearner,regr.randomForest)
S3method(trainLearner,regr.ranger)
S3method(trainLearner,regr.rpart)
S3method(trainLearner,regr.rsm)
S3method(trainLearner,regr.rvm)
S3method(trainLearner,regr.svm)
S3method(trainLearner,regr.xgboost)
S3method(trainLearner,surv.cforest)
S3method(trainLearner,surv.coxph)
S3method(trainLearner,surv.cvglmnet)
S3method(trainLearner,surv.gamboost)
S3method(trainLearner,surv.gbm)
S3method(trainLearner,surv.glmboost)
S3method(trainLearner,surv.glmnet)
S3method(trainLearner,surv.ranger)
S3method(trainLearner,surv.rpart)
export(G1)
export(G2)
export(acc)
export(addRRMeasure)
export(analyzeFeatSelResult)
export(asROCRPrediction)
export(auc)
export(b632)
export(b632plus)
export(bac)
export(batchmark)
export(benchmark)
export(ber)
export(bootstrapB632)
export(bootstrapB632plus)
export(bootstrapOOB)
export(brier)
export(brier.scaled)
export(calculateConfusionMatrix)
export(calculateROCMeasures)
export(capLargeValues)
export(changeData)
export(checkLearner)
export(checkPredictLearnerOutput)
export(cindex)
export(cindex.uno)
export(configureMlr)
export(convertBMRToRankMatrix)
export(convertMLBenchObjToTask)
export(createDummyFeatures)
export(createSpatialResamplingPlots)
export(crossval)
export(cv10)
export(cv2)
export(cv3)
export(cv5)
export(db)
export(deleteCacheDir)
export(downsample)
export(dropFeatures)
export(estimateRelativeOverfitting)
export(estimateResidualVariance)
export(expvar)
export(extractFDABsignal)
export(extractFDADTWKernel)
export(extractFDAFPCA)
export(extractFDAFeatures)
export(extractFDAFourier)
export(extractFDAMultiResFeatures)
export(extractFDATsfeatures)
export(extractFDAWavelets)
export(f1)
export(fdr)
export(featperc)
export(filterFeatures)
export(fixedcv)
export(fn)
export(fnr)
export(fp)
export(fpr)
export(friedmanPostHocTestBMR)
export(friedmanTestBMR)
export(generateCalibrationData)
export(generateCritDifferencesData)
export(generateFeatureImportanceData)
export(generateFilterValuesData)
export(generateHyperParsEffectData)
export(generateLearningCurveData)
export(generatePartialDependenceData)
export(generateThreshVsPerfData)
export(getBMRAggrPerformances)
export(getBMRFeatSelResults)
export(getBMRFilteredFeatures)
export(getBMRLearnerIds)
export(getBMRLearnerShortNames)
export(getBMRLearners)
export(getBMRMeasureIds)
export(getBMRMeasures)
export(getBMRModels)
export(getBMRPerformances)
export(getBMRPredictions)
export(getBMRTaskDescriptions)
export(getBMRTaskDescs)
export(getBMRTaskIds)
export(getBMRTuneResults)
export(getCacheDir)
export(getCaretParamSet)
export(getClassWeightParam)
export(getConfMatrix)
export(getDefaultMeasure)
export(getFailureModelDump)
export(getFailureModelMsg)
export(getFeatSelResult)
export(getFeatureImportance)
export(getFeatureImportanceLearner)
export(getFilteredFeatures)
export(getFunctionalFeatures)
export(getHomogeneousEnsembleModels)
export(getHyperPars)
export(getLearnerId)
export(getLearnerModel)
export(getLearnerNote)
export(getLearnerPackages)
export(getLearnerParVals)
export(getLearnerParamSet)
export(getLearnerPredictType)
export(getLearnerProperties)
export(getLearnerShortName)
export(getLearnerType)
export(getMeasureProperties)
export(getMlrOptions)
export(getMultilabelBinaryPerformances)
export(getNestedTuneResultsOptPathDf)
export(getNestedTuneResultsX)
export(getOOBPreds)
export(getOOBPredsLearner)
export(getPredictionDump)
export(getPredictionProbabilities)
export(getPredictionResponse)
export(getPredictionSE)
export(getPredictionTaskDesc)
export(getPredictionTruth)
export(getProbabilities)
export(getRRDump)
export(getRRPredictionList)
export(getRRPredictions)
export(getRRTaskDesc)
export(getRRTaskDescription)
export(getResamplingIndices)
export(getStackedBaseLearnerPredictions)
export(getTaskClassLevels)
export(getTaskCosts)
export(getTaskData)
export(getTaskDesc)
export(getTaskDescription)
export(getTaskFeatureNames)
export(getTaskFormula)
export(getTaskId)
export(getTaskNFeats)
export(getTaskSize)
export(getTaskTargetNames)
export(getTaskTargets)
export(getTaskType)
export(getTuneResult)
export(getTuneResultOptPath)
export(gmean)
export(gpr)
export(growingcv)
export(hasFunctionalFeatures)
export(hasLearnerProperties)
export(hasMeasureProperties)
export(hasProperties)
export(helpLearner)
export(helpLearnerParam)
export(holdout)
export(hout)
export(iauc.uno)
export(ibrier)
export(impute)
export(imputeConstant)
export(imputeHist)
export(imputeLearner)
export(imputeMax)
export(imputeMean)
export(imputeMedian)
export(imputeMin)
export(imputeMode)
export(imputeNormal)
export(imputeUniform)
export(instantiateResampleInstance)
export(isFailureModel)
export(joinClassLevels)
export(kappa)
export(kendalltau)
export(learnerArgsToControl)
export(listFilterEnsembleMethods)
export(listFilterMethods)
export(listLearnerProperties)
export(listLearners)
export(listMeasureProperties)
export(listMeasures)
export(listTaskTypes)
export(logloss)
export(lsr)
export(mae)
export(makeAggregation)
export(makeBaggingWrapper)
export(makeBaseWrapper)
export(makeChainModel)
export(makeClassifTask)
export(makeClassifTaskDesc)
export(makeClassificationViaRegressionWrapper)
export(makeClusterTask)
export(makeClusterTaskDesc)
export(makeConstantClassWrapper)
export(makeCostMeasure)
export(makeCostSensClassifWrapper)
export(makeCostSensRegrWrapper)
export(makeCostSensTask)
export(makeCostSensTaskDesc)
export(makeCostSensWeightedPairsWrapper)
export(makeCustomResampledMeasure)
export(makeDownsampleWrapper)
export(makeDummyFeaturesWrapper)
export(makeExtractFDAFeatMethod)
export(makeExtractFDAFeatsWrapper)
export(makeFeatSelControlExhaustive)
export(makeFeatSelControlGA)
export(makeFeatSelControlRandom)
export(makeFeatSelControlSequential)
export(makeFeatSelWrapper)
export(makeFilter)
export(makeFilterEnsemble)
export(makeFilterWrapper)
export(makeFixedHoldoutInstance)
export(makeFunctionalData)
export(makeImputeMethod)
export(makeImputeWrapper)
export(makeLearner)
export(makeLearners)
export(makeMeasure)
export(makeModelMultiplexer)
export(makeModelMultiplexerParamSet)
export(makeMulticlassWrapper)
export(makeMultilabelBinaryRelevanceWrapper)
export(makeMultilabelClassifierChainsWrapper)
export(makeMultilabelDBRWrapper)
export(makeMultilabelNestedStackingWrapper)
export(makeMultilabelStackingWrapper)
export(makeMultilabelTask)
export(makeMultilabelTaskDesc)
export(makeOverBaggingWrapper)
export(makeOversampleWrapper)
export(makePrediction)
export(makePreprocWrapper)
export(makePreprocWrapperCaret)
export(makeRLearner)
export(makeRLearnerClassif)
export(makeRLearnerCluster)
export(makeRLearnerCostSens)
export(makeRLearnerMultilabel)
export(makeRLearnerRegr)
export(makeRLearnerSurv)
export(makeRegrTask)
export(makeRegrTaskDesc)
export(makeRemoveConstantFeaturesWrapper)
export(makeResampleDesc)
export(makeResampleInstance)
export(makeSMOTEWrapper)
export(makeStackedLearner)
export(makeSurvTask)
export(makeSurvTaskDesc)
export(makeTaskDescInternal)
export(makeTuneControlCMAES)
export(makeTuneControlDesign)
export(makeTuneControlGenSA)
export(makeTuneControlGrid)
export(makeTuneControlIrace)
export(makeTuneControlMBO)
export(makeTuneControlRandom)
export(makeTuneMultiCritControlGrid)
export(makeTuneMultiCritControlMBO)
export(makeTuneMultiCritControlNSGA2)
export(makeTuneMultiCritControlRandom)
export(makeTuneWrapper)
export(makeUndersampleWrapper)
export(makeWeightedClassesWrapper)
export(makeWrappedModel)
export(mape)
export(mcc)
export(mcp)
export(meancosts)
export(measureACC)
export(measureAU1P)
export(measureAU1U)
export(measureAUC)
export(measureAUNP)
export(measureAUNU)
export(measureBAC)
export(measureBER)
export(measureBrier)
export(measureBrierScaled)
export(measureEXPVAR)
export(measureF1)
export(measureFDR)
export(measureFN)
export(measureFNR)
export(measureFP)
export(measureFPR)
export(measureGMEAN)
export(measureGPR)
export(measureKAPPA)
export(measureKendallTau)
export(measureLSR)
export(measureLogloss)
export(measureMAE)
export(measureMAPE)
export(measureMCC)
export(measureMEDAE)
export(measureMEDSE)
export(measureMMCE)
export(measureMSE)
export(measureMSLE)
export(measureMulticlassBrier)
export(measureMultilabelACC)
export(measureMultilabelF1)
export(measureMultilabelHamloss)
export(measureMultilabelPPV)
export(measureMultilabelSubset01)
export(measureMultilabelTPR)
export(measureNPV)
export(measurePPV)
export(measureQSR)
export(measureRAE)
export(measureRMSE)
export(measureRMSLE)
export(measureRRSE)
export(measureRSQ)
export(measureSAE)
export(measureSSE)
export(measureSSR)
export(measureSpearmanRho)
export(measureTN)
export(measureTNR)
export(measureTP)
export(measureTPR)
export(measureWKAPPA)
export(medae)
export(medse)
export(mergeBenchmarkResults)
export(mergeSmallFactorLevels)
export(mmce)
export(mse)
export(msle)
export(multiclass.au1p)
export(multiclass.au1u)
export(multiclass.aunp)
export(multiclass.aunu)
export(multiclass.brier)
export(multilabel.acc)
export(multilabel.f1)
export(multilabel.hamloss)
export(multilabel.ppv)
export(multilabel.subset01)
export(multilabel.tpr)
export(normalizeFeatures)
export(npv)
export(oversample)
export(performance)
export(plotBMRBoxplots)
export(plotBMRRanksAsBarChart)
export(plotBMRSummary)
export(plotCalibration)
export(plotCritDifferences)
export(plotFilterValues)
export(plotHyperParsEffect)
export(plotLearnerPrediction)
export(plotLearningCurve)
export(plotPartialDependence)
export(plotROCCurves)
export(plotResiduals)
export(plotThreshVsPerf)
export(plotTuneMultiCritResult)
export(ppv)
export(predictLearner)
export(qsr)
export(rae)
export(reduceBatchmarkResults)
export(reextractFDAFeatures)
export(reimpute)
export(removeConstantFeatures)
export(removeHyperPars)
export(repcv)
export(resample)
export(rmse)
export(rmsle)
export(rrse)
export(rsq)
export(sae)
export(selectFeatures)
export(setAggregation)
export(setHyperPars)
export(setHyperPars2)
export(setId)
export(setLearnerId)
export(setMeasurePars)
export(setPredictThreshold)
export(setPredictType)
export(setThreshold)
export(silhouette)
export(simplifyMeasureNames)
export(smote)
export(spearmanrho)
export(sse)
export(ssr)
export(subsample)
export(subsetTask)
export(summarizeColumns)
export(summarizeLevels)
export(test.join)
export(test.max)
export(test.mean)
export(test.median)
export(test.min)
export(test.range)
export(test.rmse)
export(test.sd)
export(test.sum)
export(testgroup.mean)
export(testgroup.sd)
export(timeboth)
export(timepredict)
export(timetrain)
export(tn)
export(tnr)
export(tp)
export(tpr)
export(train)
export(train.max)
export(train.mean)
export(train.median)
export(train.min)
export(train.range)
export(train.rmse)
export(train.sd)
export(train.sum)
export(trainLearner)
export(tuneParams)
export(tuneParamsMultiCrit)
export(tuneThreshold)
export(undersample)
export(wkappa)
import(BBmisc)
import(ParamHelpers)
import(checkmate)
import(data.table)
import(ggplot2)
import(methods)
import(parallelMap)
import(stats)
import(stringi)
importFrom(data.table,data.table)
importFrom(data.table,melt)
importFrom(graphics,hist)
importFrom(survival,Surv)
importFrom(survival,is.Surv)
importFrom(utils,adist)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,getFromNamespace)
importFrom(utils,getS3method)
importFrom(utils,head)
importFrom(utils,methods)
importFrom(utils,tail)
importFrom(utils,type.convert)
useDynLib(mlr,c_smote)
