#' @title List of supported learning algorithms.
#'
#' @description
#' All supported learners can be found by [listLearners] or as a table
#' in the tutorial appendix: <https://mlr.mlr-org.com/articles/tutorial/integrated_learners.html>.
#'
#' @name learners
#' @rdname learners
NULL
