% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{iris.task}
\alias{iris.task}
\title{Iris classification task.}
\description{
Contains the task (\code{iris.task}).
}
\references{
See \link[datasets:iris]{datasets::iris}.
}
\keyword{data}
